'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _events = require('events');

var _pcsclite = require('pcsclite');

var _pcsclite2 = _interopRequireDefault(_pcsclite);

var _hexify = require('hexify');

var _hexify2 = _interopRequireDefault(_hexify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var devices = new _events.EventEmitter();
var pcsc = (0, _pcsclite2.default)();
var cardReader = void 0;
var protocol = void 0;

var cardInserted = function cardInserted(reader, status) {
    reader.connect(function (err, readerProtocol) {
        if (err) {
            devices.emit('error', err);
        } else {
            cardReader = reader;
            protocol = readerProtocol;
            devices.emit('debug', 'Device \'' + reader.name + '\' has protocol \'' + protocol + '\'');
            devices.emit('card-inserted', reader, status);
        }
    });
};

var cardRemoved = function cardRemoved(reader) {
    reader.disconnect(reader.SCARD_LEAVE_CARD, function (err) {
        if (err) {
            devices.emit('error', err);
        } else {
            if (cardReader) {
                devices.emit('card-removed', reader);
            }
        }
    });
    cardReader = null;
    protocol = null;
};

var isCardInserted = function isCardInserted(changes, reader, status) {
    return changes & reader.SCARD_STATE_PRESENT && status.state & reader.SCARD_STATE_PRESENT;
};

var isCardRemoved = function isCardRemoved(changes, reader, status) {
    return changes & reader.SCARD_STATE_EMPTY && status.state & reader.SCARD_STATE_EMPTY;
};

var deviceActivated = function deviceActivated(reader) {
    devices.emit('device-activated', reader);

    reader.on('status', function (status) {
        var changes = reader.state ^ status.state;
        if (changes) {
            if (isCardRemoved(changes, reader, status)) {
                cardRemoved(reader);
            } else if (isCardInserted(changes, reader, status)) {
                cardInserted(reader, status);
            }
        }
    });

    reader.on('end', function () {
        devices.emit('device-deactivated', reader);
    });

    reader.on('error', function (err) {
        devices.emit('error', err);
    });
};

pcsc.on('reader', function (reader) {
    deviceActivated(reader);
});

pcsc.on('error', function (err) {
    devices.emit('error', err);
});

devices.issueCommand = function (command, callback) {
    var buffer = void 0;
    if (Array.isArray(command)) {
        buffer = new Buffer(command);
    } else if (typeof command === 'string') {
        buffer = new Buffer(_hexify2.default.toByteArray(command));
    } else if (Buffer.isBuffer(command)) {
        buffer = command;
    } else {
        throw 'Unable to recognise command type (' + (typeof command === 'undefined' ? 'undefined' : _typeof(command)) + ')';
    }

    devices.emit('issue-command', cardReader, buffer);
    if (callback) {
        cardReader.transmit(buffer, 0xFF, protocol, function (err, response) {
            devices.emit('receive-response', cardReader, new Buffer(response.toString('hex')));
            callback(err, response);
        });
    } else {
        return new Promise(function (resolve, reject) {
            cardReader.transmit(buffer, 0xFF, protocol, function (err, response) {
                if (err) reject(err);else {
                    devices.emit('receive-response', cardReader, new Buffer(response.toString('hex')), new Buffer(command.toLowerCase()));
                    resolve(response);
                }
            });
        });
    }
};

module.exports = devices;